﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Drawing.Imaging;
using System.IO;
using System.Drawing.Drawing2D;

namespace howto_clock_2023
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        Bitmap[] Hours = null;

        private void mnuFileOpen_Click(object sender, EventArgs e)
        {
            if (ofdCake.ShowDialog() == DialogResult.OK)
            {
                Hours = new Bitmap[12];
                int i = 0;
                foreach (string filename in ofdCake.FileNames)
                {
                    if (i >= 12) break;
                    Hours[i++] = LoadBitmapUnlocked(filename);
                }
                DrawClock();
            }
        }

        private void mnuFileSaveAs_Click(object sender, EventArgs e)
        {
            if (sfdCake.ShowDialog() == DialogResult.OK)
            {
                SaveImage(picClock.Image, sfdCake.FileName);
            }
        }

        private void mnuFileExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void DrawClock()
        {
            const int wid = 1200;
            const int hgt = 900;
            int min_wid_hgt = Math.Min(wid, hgt);

            float cx = wid / 2f;
            float cy = hgt / 2f;

            Bitmap bm = new Bitmap(wid, hgt);
            using (Graphics gr = Graphics.FromImage(bm))
            {
                using (LinearGradientBrush brush = new LinearGradientBrush(
                    new Point(0, 0),
                    new Point(0, hgt),
                    Color.White, Color.LightGreen))
                {
                    ColorBlend blend = new ColorBlend(3);
                    blend.Colors = new Color[]
                    {
                        Color.White,
                        Color.LightGray,
                        Color.LightGreen,
                    };
                    blend.Positions = new float[]{ 0.0f, 0.66f, 1.0f };
                    brush.InterpolationColors = blend;

                    //gr.Clear(Color.White);
                    gr.FillRectangle(brush, 0, 0, wid, hgt);

                    float slice_wid = min_wid_hgt / 3.5f;
                    float radius = (min_wid_hgt - 20) / 2f - slice_wid / 2f;

                    for (int i = 0; i < 12; i++)
                    {
                        if (Hours[i] == null) break;

                        float degrees = i * 360f / 12f;
                        gr.RotateTransform(degrees + 180);

                        float scale = slice_wid / Hours[i].Width;
                        gr.ScaleTransform(scale, scale, MatrixOrder.Append);

                        double radians = degrees * Math.PI / 180;
                        float x = (float)(cx + radius * Math.Cos(radians));
                        float y = (float)(cy + radius * Math.Sin(radians));
                        gr.TranslateTransform(x, y, MatrixOrder.Append);

                        RectangleF rect = new RectangleF(
                            -Hours[i].Width / 2,
                            -Hours[i].Height / 2,
                            Hours[i].Width,
                            Hours[i].Height);
                        gr.DrawImage(Hours[i], rect);

                        gr.ResetTransform();
                    }

                    // Draw the clock hands.
                    float pen_thickness = min_wid_hgt / 50f;
                    using (Pen pen = new Pen(Color.Blue, pen_thickness))
                    {
                        pen.EndCap = LineCap.Round;
                        pen.StartCap = LineCap.Round;

                        using (Pen outline_pen = new Pen(brush, pen_thickness * 1.1f))
                        {
                            outline_pen.EndCap = LineCap.Round;
                            outline_pen.StartCap = LineCap.Round;

                            float long_len = min_wid_hgt * 0.4f;
                            double long_angle = 2 * Math.PI * 11.0 / 12.0;
                            float x1 = (float)(cx + long_len * Math.Cos(long_angle));
                            float y1 = (float)(cy + long_len * Math.Sin(long_angle));
                            gr.DrawLine(outline_pen, cx, cy, x1, y1);
                            gr.DrawLine(pen, cx, cy, x1, y1);

                            float short_len = min_wid_hgt * 0.3f;
                            double short_angle = 2 * Math.PI * 7.0 / 12.0;
                            float x2 = (float)(cx + short_len * Math.Cos(short_angle));
                            float y2 = (float)(cy + short_len * Math.Sin(short_angle));
                            gr.DrawLine(outline_pen, cx, cy, x2, y2);
                            gr.DrawLine(pen, cx, cy, x2, y2);

                            RectangleF rect = new RectangleF(
                                cx - pen_thickness,
                                cy - pen_thickness,
                                2 * pen_thickness,
                                2 * pen_thickness);

                            gr.FillEllipse(Brushes.DarkBlue, rect);
                        }
                    }
                }
            }

            picClock.Image = bm;
        }

        // Load a bitmap without locking it.
        private Bitmap LoadBitmapUnlocked(string file_name)
        {
            using (Bitmap bm = new Bitmap(file_name))
            {
                return new Bitmap(bm);
            }
        }

        // Save the file with the appropriate format.
        public void SaveImage(Image image, string filename)
        {
            string extension = Path.GetExtension(filename);
            switch (extension.ToLower())
            {
                case ".bmp":
                    image.Save(filename, ImageFormat.Bmp);
                    break;
                case ".exif":
                    image.Save(filename, ImageFormat.Exif);
                    break;
                case ".gif":
                    image.Save(filename, ImageFormat.Gif);
                    break;
                case ".jpg":
                case ".jpeg":
                    image.Save(filename, ImageFormat.Jpeg);
                    break;
                case ".png":
                    image.Save(filename, ImageFormat.Png);
                    break;
                case ".tif":
                case ".tiff":
                    image.Save(filename, ImageFormat.Tiff);
                    break;
                default:
                    throw new NotSupportedException(
                        "Unknown file extension " + extension);
            }
        }
    }
}
